@@ -5,14 +5,15 @@ use warnings;
 use FindBin;
 use File::Basename;
 use Getopt::Long qw(:config no_ignore_case bundling pass_through);
+use Carp;
 
 my $usage = <<__EOUSAGE__;
 
-############################################################
+####################################################################################
 #
 # Usage:  $0 --est_method <method>  sample1.results sample2.results ...
 #
-#      or  $0 --est_method <method> --samples_file file.listing_target_files.txt
+#      or  $0 --est_method <method> --quant_files file.listing_target_files.txt
 #
 #      Note, if only a single input file is given, it's expected to contain the paths to all the target abundance estimation files.
 #
@@ -20,6 +21,9 @@ my $usage = <<__EOUSAGE__;
 #            
 #  --est_method <string>           RSEM|eXpress|kallisto|salmon  (needs to know what format to expect)
 #
+#  --gene_trans_map <string>           the gene-to-transcript mapping file. (if you don't want gene estimates, indicate 'none'.
+#
+#
 # Options:
 #
 #  --cross_sample_norm <string>         TMM|UpperQuartile|none   (default: TMM)
@@ -27,11 +31,11 @@ my $usage = <<__EOUSAGE__;
 #  --name_sample_by_basedir             name sample column by dirname instead of filename
 #      --basedir_index <int>            default(-2)
 #
-#  --out_prefix <string>                default: 'matrix'
+#  --out_prefix <string>                default: value for --est_method
 #
-#  --samples_file <string>              file containing a list of all the target files.
+#  --quant_files <string>              file containing a list of all the target files.
 #
-############################################################
+######################################################################################
 
 
 __EOUSAGE__
@@ -44,9 +48,10 @@ my $est_method;
 my $val_type;
 my $cross_sample_norm = "TMM";
 my $name_sample_by_basedir = 0;
-my $out_prefix = "matrix";
+my $out_prefix;
 my $basedir_index = -2;
-my $samples_file = "";
+my $quant_files = "";
+my $gene_trans_map_file;
 
 &GetOptions('help|h' => \$help_flag,
             'est_method=s' => \$est_method,
@@ -56,14 +61,22 @@ my $samples_file = "";
             'out_prefix=s' => \$out_prefix,
             'basedir_index=i' => \$basedir_index,
             
-            'samples_file=s' => \$samples_file,
-            );
+            'quant_files=s' => \$quant_files,
+            'gene_trans_map=s' => \$gene_trans_map_file,
+    );
 
 
-unless ($est_method && @ARGV) {
+
+if ($help_flag) { die $usage; }
+
+unless ($est_method && (@ARGV || $quant_files)) {
     die $usage;
 }
 
+unless ($gene_trans_map_file) {
+    die "Error, specify gene-to-trans map file via: --gene_trans_map, or indicate 'none' if you dont want gene estimates";
+}
+
 unless ($est_method =~ /^(RSEM|eXpress|kallisto|salmon)/i) {
     die "Error, dont recognize --est_method $est_method ";
 }
@@ -71,11 +84,15 @@ unless ($cross_sample_norm =~ /^(TMM|UpperQuartile|none)$/i) {
     die "Error, dont recognize --cross_sample_norm $cross_sample_norm ";
 }
 
+unless ($out_prefix) {
+    $out_prefix = $est_method;
+}
+
 my @files;
 
-if ($samples_file) {
+if ($quant_files) {
     # allow for a file listing the various files.
-    @files = `cat $samples_file`;
+    @files = `cat $quant_files`;
     chomp @files;
 }
 elsif (@ARGV) {
@@ -171,6 +188,8 @@ else {
 main: {
     
     my %data;
+
+    my %sum_sample_counts;
     
     foreach my $file (@files) {
         print STDERR "-reading file: $file\n";
@@ -191,27 +210,37 @@ main: {
             $data{$acc}->{$file}->{count} = $count;
             $data{$acc}->{$file}->{FPKM} = $fpkm;
             $data{$acc}->{$file}->{TPM} = $tpm;
+            
+            # capture sample total counts
+            $sum_sample_counts{$file} += $count;
         }
         close $fh;
     }
-    
+
+    my %column_header_to_filename;
     my @filenames = @files;
     foreach my $file (@filenames) {
+        my $column_header;
         if ($name_sample_by_basedir) {
             my @path = split(m|/|, $file);
-            $file = $path[$basedir_index];
+            $column_header = $path[$basedir_index];
         }
         else {
-            $file = basename($file);
+            $column_header = basename($file);
         }
+        
+        $column_header =~ s/\.(genes|isoforms)\.results$//; # in case of rsem
+        
+        $column_header_to_filename{$column_header} = $file;
+        $file = $column_header; # update the @filenames
     }
     print STDERR "\n\n* Outputting combined matrix.\n\n";
     
-    my $counts_matrix_file = "$out_prefix.counts.matrix";
-    my $TPM_matrix_file = "$out_prefix.TPM.not_cross_norm";
+    my $counts_matrix_file = "$out_prefix.isoform.counts.matrix";
+    my $TPM_matrix_file = "$out_prefix.isoform.TPM.not_cross_norm";
     open (my $ofh_counts, ">$counts_matrix_file") or die "Error, cannot write file $counts_matrix_file";
     open (my $ofh_TPM, ">$TPM_matrix_file") or die "Error, cannot write file $TPM_matrix_file";
-
+    
     { # check to see if they're unique
         my %filename_map = map { + $_ => 1 } @filenames;
         if (scalar keys %filename_map != scalar @filenames) {
@@ -221,10 +250,10 @@ main: {
     
 
     # clean up matrix headers
-    foreach my $file (@filenames) {
+    #foreach my $file (@filenames) {
         # also, get rid of the part of the filename that RSEM adds
-        $file =~ s/\.(genes|isoforms)\.results$//;
-    }
+    #    $file =~ s/\.(genes|isoforms)\.results$//;
+    #}
     
 
     print $ofh_counts join("\t", "", @filenames) . "\n";
@@ -259,34 +288,93 @@ main: {
     }
     close $ofh_counts;
     close $ofh_TPM;
-    
 
+    ## process gene counts as per txImport-style (Soneson et al. F1000, 2016)
+    my $gene_counts_file = "$out_prefix.gene.counts.matrix";
+    my $gene_tpm_file = "$out_prefix.gene.TPM.not_cross_norm";
+    
+    if ($gene_trans_map_file ne 'none') {
+        my %gene_to_trans;
+        {
+            open(my $fh, $gene_trans_map_file) or die "Error, cannot open file $gene_trans_map_file";
+            while (<$fh>) {
+                chomp;
+                my ($gene, $trans) = split(/\s+/);
+                push (@{$gene_to_trans{$gene}}, $trans);
+            }
+            close $fh;
+        }
+
+        open(my $ofh_genecounts, ">$gene_counts_file") or die "Error, cannot write to $gene_counts_file";
+        print $ofh_genecounts "\t" . join("\t", @filenames) . "\n";
+        
+        open(my $ofh_genetpm, ">$gene_tpm_file") or die "Error, cannot write to $gene_tpm_file";
+        print $ofh_genetpm "\t" . join("\t", @filenames) . "\n";
+        
+        foreach my $gene (sort keys %gene_to_trans) {
+            my @tpm_vals = ($gene);
+            my @count_vals = ($gene);
+            foreach my $file (@filenames) {
+                my $gene_tpm = 0;
+                # sum up gene tpm from isoform tpms
+                foreach my $trans (@{$gene_to_trans{$gene}}) {
+                    my $trans_tpm = $data{$trans}->{ $column_header_to_filename{$file} }->{TPM};
+                    unless (defined $trans_tpm) {
+                        confess "Error, no TPM value specified for transcript [$trans] of gene [$gene] for sample $file";
+                    }
+                    $gene_tpm += $trans_tpm;
+                }
+                push (@tpm_vals, $gene_tpm);
+                my $gene_count = $gene_tpm / 1e6 * $sum_sample_counts{ $column_header_to_filename{$file} };
+                $gene_count = sprintf("%.2f", $gene_count);
+                push (@count_vals, $gene_count);
+            }
+            
+            print $ofh_genetpm join("\t", @tpm_vals) . "\n";
+            print $ofh_genecounts join("\t", @count_vals) . "\n";
+        }
+        close $ofh_genetpm;
+        close $ofh_genecounts;
+    }
     if (scalar @files > 1) {
         ## more than one sample 
         
-        if ($cross_sample_norm =~ /^TMM$/i) {
-            my $cmd = "$FindBin::RealBin/support_scripts/run_TMM_scale_matrix.pl --matrix $TPM_matrix_file > $out_prefix.$cross_sample_norm.EXPR.matrix";
-            &process_cmd($cmd);
-        }
-        elsif ($cross_sample_norm =~ /^UpperQuartile$/) {
-            my $cmd = "$FindBin::RealBin/support_scripts/run_UpperQuartileNormalization_matrix.pl --matrix $TPM_matrix_file > $out_prefix.$cross_sample_norm.EXPR.matrix";
-            &process_cmd($cmd);
-        }
-        elsif ($cross_sample_norm =~ /^none$/i) {
-            print STDERR "-not performing cross-sample normalization.\n";
+        &perform_cross_sample_norm($TPM_matrix_file, "$out_prefix.isoform");
+        if ($gene_trans_map_file ne 'none') {
+            &perform_cross_sample_norm($gene_tpm_file, "$out_prefix.gene");
         }
+        
     }
     else {
         unless (scalar @files == 1) { 
             die "Error, no target samples. Shouldn't get here.";
         }
         print STDERR "Warning, only one sample, so not performing cross-sample normalization\n";
+        print STDERR "Done.\n\n";
     }
-    print STDERR "Done.\n\n";
     
     exit(0);
 }
 
+
+####
+sub perform_cross_sample_norm {
+    my ($tpm_matrix_file, $out_prefix_name) = @_;
+    
+    if ($cross_sample_norm =~ /^TMM$/i) {
+        my $cmd = "$FindBin::RealBin/support_scripts/run_TMM_scale_matrix.pl --matrix $tpm_matrix_file > $out_prefix_name.$cross_sample_norm.EXPR.matrix";
+        &process_cmd($cmd);
+    }
+    elsif ($cross_sample_norm =~ /^UpperQuartile$/) {
+        my $cmd = "$FindBin::RealBin/support_scripts/run_UpperQuartileNormalization_matrix.pl --matrix $tpm_matrix_file > $out_prefix_name.$cross_sample_norm.EXPR.matrix";
+        &process_cmd($cmd);
+    }
+    elsif ($cross_sample_norm =~ /^none$/i) {
+        print STDERR "-not performing cross-sample normalization.\n";
+    }
+
+}
+
 ####
 sub process_cmd {
     my ($cmd) = @_;
