@@ -13,7 +13,7 @@ my $file = $ARGV[0] or die $usage;
 
 my $fasta_reader = new Fasta_reader($file);
 
-print join("\t", "fasta_entry", "length") . "\n";
+print join("\t", "#fasta_entry", "length") . "\n";
 while (my $seq_obj = $fasta_reader->next()) {
     my $sequence = $seq_obj->get_sequence();
     my $accession = $seq_obj->get_accession();
